# SPDX-FileCopyrightText: 2022-present Manuel Quarneti <manuelquarneti@gmail.com>
#
# SPDX-License-Identifier: GPL-3.0-only

import json
import subprocess
from os import listdir, makedirs, path
from typing import List, TypedDict

from minecraft_launcher_lib.command import get_minecraft_command
from minecraft_launcher_lib.install import install_minecraft_version
from minecraft_launcher_lib.runtime import get_executable_path
from minecraft_launcher_lib.types import MinecraftOptions

from . import accounts, dirs

__instances_dir__: str = path.join(dirs.user_data_dir, "instances")


class InstanceJson(TypedDict):
    config_version: int
    minecraft_version: str


def list() -> List[str]:
    # check if instances folder exists
    if not path.exists(__instances_dir__):
        makedirs(__instances_dir__)

    list = listdir(__instances_dir__)
    if ".DS_Store" in list:
        list.remove(".DS_Store")

    return list


def new(instance_name: str, minecraft_version: str) -> None:
    print("Creating instance")
    instance_dir = path.join(__instances_dir__, instance_name)
    makedirs(instance_dir)
    instance_json: InstanceJson = {
        "config_version": 1,
        "minecraft_version": minecraft_version,
    }
    with open(path.join(instance_dir, "instance.json"), "w") as f:
        json.dump(instance_json, f)

    install_minecraft_version(minecraft_version, dirs.user_data_dir)
    print("Done")


def get_info(instance_name: str) -> InstanceJson:
    with open(path.join(__instances_dir__, instance_name, "instance.json"), "r") as f:
        return json.load(f)


def launch(instance_name: str, account_name: str, launcher_version: str) -> None:
    account = next(
        a for a in accounts.read_document()["accounts"] if a["name"] == account_name
    )

    instance_dir = path.join(__instances_dir__, instance_name)
    with open(path.join(instance_dir, "instance.json"), "r") as f:
        instance_json: InstanceJson = json.load(f)

    with open(
        path.join(
            dirs.user_data_dir,
            "versions",
            instance_json["minecraft_version"],
            f"{instance_json['minecraft_version']}.json",
        ),
        "r",
    ) as f:
        version_json: dict = json.load(f)

    jvm_version = version_json["javaVersion"]["component"]
    java_executable = get_executable_path(jvm_version, dirs.user_data_dir)
    if java_executable is None:
        java_executable = "java"

    options: MinecraftOptions = {
        "username": account["name"],
        "uuid": account["id"],
        "token": account["access_token"],
        "executablePath": java_executable,
        "jvmArguments": ["-Xmx2G", "-Xms2G"],
        "launcherName": "Ice Launcher",
        "launcherVersion": launcher_version,
        "gameDirectory": path.join(__instances_dir__, instance_name),
    }

    minecraft_command = get_minecraft_command(
        instance_json["minecraft_version"], dirs.user_data_dir, options
    )
    subprocess.call(minecraft_command)



# SPDX-FileCopyrightText: 2022-present Manuel Quarneti <manuelquarneti@gmail.com>
#
# SPDX-License-Identifier: GPL-3.0-only

from typing import List

from minecraft_launcher_lib.types import MinecraftVersionInfo
from minecraft_launcher_lib.utils import get_available_versions

from . import dirs


def get_available() -> List[MinecraftVersionInfo]:
    return get_available_versions(dirs.user_data_dir)



# SPDX-FileCopyrightText: 2022-present Manuel Quarneti <manuelquarneti@gmail.com>
#
# SPDX-License-Identifier: GPL-3.0-only

# Based on https://gist.github.com/mp035/9f2027c3ef9172264532fcd6262f3b01

import platform
from tkinter import Canvas

from customtkinter import CTkFrame, CTkScrollbar


class ScrollableFrame(CTkFrame):
    def __init__(self, master):
        super().__init__(master)  # create a frame (self)

        self.grid_rowconfigure(
            0, weight=1
        )  # make the frame expandable in the y direction
        self.grid_columnconfigure(
            0, weight=1
        )  # make the frame expandable in the x direction

        self.canvas = Canvas(
            self, borderwidth=0, background="#2A2D2E", highlightthickness=0
        )  # place canvas on self
        self.content = CTkFrame(
            self.canvas
        )  # place a frame on the canvas, this frame will hold the child widgets
        self.scrollbar = CTkScrollbar(
            self, command=self.canvas.yview
        )  # place a scrollbar on self
        self.canvas.configure(
            yscrollcommand=self.scrollbar.set
        )  # attach scrollbar action to scroll of canvas

        self.scrollbar.grid(row=0, column=1, padx=5, pady=5, sticky="ns")
        self.canvas.grid(row=0, column=0, sticky="nsew")  # attach canvas to self
        self.canvas_window = self.canvas.create_window(
            (0, 0),
            window=self.content,
            anchor="nw",  # add view port frame to canvas
            tags="self.viewPort",
        )

        self.content.bind(
            "<Configure>", self.on_frame_configure
        )  # bind an event whenever the size of the viewPort frame changes.
        self.canvas.bind(
            "<Configure>", self.on_canvas_configure
        )  # bind an event whenever the size of the canvas frame changes.

        self.content.bind(
            "<Enter>", self.on_enter
        )  # bind wheel events when the cursor enters the control
        self.content.bind(
            "<Leave>", self.on_leave
        )  # unbind wheel events when the cursorl leaves the control

        self.on_frame_configure(
            None
        )  # perform an initial stretch on render, otherwise the scroll region has a tiny border until the first resize

    def on_frame_configure(self, event):
        """Reset the scroll region to encompass the inner frame"""
        self.canvas.configure(
            scrollregion=self.canvas.bbox("all")
        )  # whenever the size of the frame changes, alter the scroll region respectively.

    def on_canvas_configure(self, event):
        """Reset the canvas window to encompass inner frame when required"""
        canvas_width = event.width
        self.canvas.itemconfig(
            self.canvas_window, width=canvas_width
        )  # whenever the size of the canvas changes alter the window region respectively.

    def on_mouse_wheel(self, event):  # cross platform scroll wheel event
        if platform.system() == "Windows":
            self.canvas.yview_scroll(int(-1 * (event.delta / 120)), "units")
        elif platform.system() == "Darwin":
            self.canvas.yview_scroll(int(-1 * event.delta), "units")
        else:
            if event.num == 4:
                self.canvas.yview_scroll(-1, "units")
            elif event.num == 5:
                self.canvas.yview_scroll(1, "units")

    def on_enter(self, event):  # bind wheel events when the cursor enters the control
        if platform.system() == "Linux":
            self.canvas.bind_all("<Button-4>", self.on_mouse_wheel)
            self.canvas.bind_all("<Button-5>", self.on_mouse_wheel)
        else:
            self.canvas.bind_all("<MouseWheel>", self.on_mouse_wheel)

    def on_leave(
        self, event
    ):  # unbind wheel events when the cursorl leaves the control
        if platform.system() == "Linux":
            self.canvas.unbind_all("<Button-4>")
            self.canvas.unbind_all("<Button-5>")
        else:
            self.canvas.unbind_all("<MouseWheel>")



# SPDX-FileCopyrightText: 2022-present Manuel Quarneti <manuelquarneti@gmail.com>
#
# SPDX-License-Identifier: GPL-3.0-only

import webbrowser
from http.server import BaseHTTPRequestHandler, HTTPServer
from threading import Thread
from tkinter import ttk

from customtkinter import CTkButton, CTkFrame, CTkLabel
from minecraft_launcher_lib import microsoft_account as msa

from .components.scrollable_frame import ScrollableFrame
from .lib import accounts

__client_id__: str = "0018ddff-bd2f-4cc6-b220-66f6a4462a5c"
__redirect_uri__: str = "http://localhost:3003"


class CallbackHandler(BaseHTTPRequestHandler):
    state: str
    code_verifier: str

    def do_GET(self) -> None:
        auth_code = msa.parse_auth_code_url(self.path, self.state)

        login_data = msa.complete_login(
            client_id=__client_id__,
            client_secret=None,
            redirect_uri=__redirect_uri__,
            auth_code=auth_code,
            code_verifier=self.code_verifier,
        )

        doc = accounts.read_document()
        doc["accounts"].append(login_data)
        accounts.write_document(doc)

        self.send_response(200)
        self.send_header("Content-type", "text/html")
        self.end_headers()
        self.wfile.write(b"<html><head><title>Login</title></head>")
        self.wfile.write(b"<body><p>You have been logged in.</p>")
        self.wfile.write(b"<p>You can close this window.</p>")
        self.wfile.write(b"</body></html>")

        Thread(target=self.server.shutdown).start()


class Accounts(CTkFrame):
    doc: accounts.Document

    def __init__(self, master) -> None:
        super().__init__(master=master)

        self.grid_columnconfigure(0, weight=1)

        self.doc = accounts.read_document()

        self.title_frame = CTkFrame(master=self, fg_color="gray38")
        self.title_frame.grid(row=0, column=0, pady=20, padx=20, sticky="nswe")

        self.view_name = CTkLabel(
            master=self.title_frame,
            text_font=("Roboto Medium", 30),
            text="Accounts",
        )
        self.view_name.grid(row=0, column=0, pady=20, padx=20, sticky="nswe")

        self.accounts_list = ScrollableFrame(master=self)
        self.accounts_list.grid(row=1, column=0, pady=20, padx=20, sticky="nswe")
        self.grid_rowconfigure(1, weight=1)
        self.accounts_list.content.grid_columnconfigure(0, weight=1)

        self.update_accounts_list()

    def add_account(self) -> None:
        login_url, state, code_verifier = msa.get_secure_login_data(
            __client_id__, __redirect_uri__
        )

        webbrowser.open(login_url)
        handler = CallbackHandler
        handler.state = state
        handler.code_verifier = code_verifier
        httpd = HTTPServer(("127.0.0.1", 3003), handler)
        httpd.serve_forever()
        self.update_accounts_list()

    def delete_account(self, index) -> None:
        del self.doc["accounts"][index]
        accounts.write_document(self.doc)
        self.update_accounts_list()

    def update_accounts_list(self) -> None:
        for account in self.accounts_list.content.winfo_children():
            account.destroy()

        for index, account in enumerate(self.doc["accounts"]):
            label = CTkLabel(master=self.accounts_list.content, text=account["name"])
            label.grid(row=index * 2, column=0, pady=10, padx=0, sticky="nw")
            delete_button = CTkButton(
                master=self.accounts_list.content,
                text="Delete",
                command=lambda index=index: self.delete_account(index),
            )
            delete_button.grid(row=index * 2, column=1, pady=10, padx=10, sticky="e")
            separator = ttk.Separator(self.accounts_list.content, orient="horizontal")
            separator.grid(
                row=index * 2 + 1, column=0, columnspan=2, pady=0, padx=10, sticky="ew"
            )

        add_account_button = CTkButton(
            master=self.accounts_list.content,
            text="Add Account",
            command=self.add_account,
        )
        add_account_button.grid(
            row=len(self.doc["accounts"]) * 2, column=0, pady=20, padx=20, sticky="nw"
        )



# SPDX-FileCopyrightText: 2022-present Manuel Quarneti <manuelquarneti@gmail.com>
#
# SPDX-License-Identifier: GPL-3.0-only

from typing import Optional

import requests
from packaging import version


def _get_latest_release() -> str:
    r = requests.get("https://api.github.com/repos/mq1/ice-launcher/releases/latest")
    return r.json()["tag_name"]


def check_for_updates(current_version: str) -> Optional[str]:
    if "dev" in current_version:
        return None

    current_release = version.parse(current_version)
    latest_release = version.parse(_get_latest_release())

    return latest_release.__str__() if latest_release > current_release else None



# SPDX-FileCopyrightText: 2022-present Manuel Quarneti <manuelquarneti@gmail.com>
#
# SPDX-License-Identifier: GPL-3.0-only

from customtkinter import CTkButton, CTkComboBox, CTkEntry, CTkToplevel, StringVar
from minecraft_launcher_lib.utils import get_latest_version

from .lib import instances, versions


class NewInstance(CTkToplevel):
    def __init__(self, master) -> None:
        super().__init__(master=master)
        self.title("Add new instance")
        self.geometry("250x200")

        self.versions = versions.get_available()
        self.version_ids = [version["id"] for version in self.versions]

        self.grid_columnconfigure(0, weight=1)

        self.instance_name = CTkEntry(master=self, placeholder_text="My new instance")
        self.instance_name.grid(row=0, column=0, pady=(20, 10), padx=20, sticky="nswe")

        self.version = StringVar(value=get_latest_version()["release"])
        self.version_selector = CTkComboBox(
            master=self, values=self.version_ids, variable=self.version
        )
        self.version_selector.grid(row=1, column=0, pady=10, padx=20, sticky="nswe")

        # empty row as spacing
        self.grid_rowconfigure(2, weight=1)

        self.create_button = CTkButton(
            master=self,
            text="Create",
            command=self.create_instance,
        )
        self.create_button.grid(row=3, column=0, pady=(10, 20), padx=20)

    def create_instance(self) -> None:
        instances.new(self.instance_name.get(), self.version.get())
        self.destroy()



# SPDX-FileCopyrightText: 2022-present Manuel Quarneti <manuelquarneti@gmail.com>
#
# SPDX-License-Identifier: GPL-3.0-only

import webbrowser
from threading import Thread
from tkinter import ttk

from customtkinter import CTkButton, CTkFrame, CTkLabel
from minecraft_launcher_lib.types import Article, Articles
from minecraft_launcher_lib.utils import get_minecraft_news

from .components.scrollable_frame import ScrollableFrame


class News(CTkFrame):
    articles: Articles

    def __init__(self, master) -> None:
        super().__init__(master=master)

        self.grid_columnconfigure(0, weight=1)

        self.title_frame = CTkFrame(master=self, fg_color="gray38")
        self.title_frame.grid(row=0, column=0, pady=20, padx=20, sticky="nswe")

        self.view_name = CTkLabel(
            master=self.title_frame,
            text_font=("Roboto Medium", 30),
            text="News",
        )
        self.view_name.grid(row=0, column=0, pady=20, padx=20, sticky="nswe")

        self.news_frame = ScrollableFrame(master=self)
        self.news_frame.grid(row=1, column=0, pady=20, padx=20, sticky="nswe")
        self.grid_rowconfigure(1, weight=1)
        self.news_frame.content.grid_columnconfigure(0, weight=1)

        Thread(target=self.update_news).start()

    def update_news(self) -> None:
        articles = get_minecraft_news()["article_grid"]

        for index, article in enumerate(articles):
            label = CTkLabel(
                master=self.news_frame.content, text=article["default_tile"]["title"]
            )
            label.grid(row=index * 2, column=0, pady=10, padx=10, sticky="nw")
            open_button = CTkButton(
                master=self.news_frame.content,
                text="Open",
                command=lambda: self.open_article(article),
            )
            open_button.grid(row=index * 2, column=1, pady=10, padx=10, sticky="e")
            separator = ttk.Separator(self.news_frame.content, orient="horizontal")
            separator.grid(
                row=index * 2 + 1, column=0, columnspan=2, pady=0, padx=10, sticky="ew"
            )

    def open_article(self, article: Article) -> None:
        url = f"https://www.minecraft.net{article['article_url']}"
        webbrowser.open(url)



# SPDX-FileCopyrightText: 2022-present Manuel Quarneti <manuelquarneti@gmail.com>
#
# SPDX-License-Identifier: GPL-3.0-only

from threading import Thread

import customtkinter
from customtkinter import CTk, CTkButton, CTkFrame

from ice_launcher import About, Accounts, Instances, News, Settings, Update
from ice_launcher.__about__ import __version__
from ice_launcher.lib import config, updater

customtkinter.set_appearance_mode("dark")
customtkinter.set_default_color_theme("blue")


class App(CTk):
    WIDTH: int = 780
    HEIGHT: int = 520

    views: dict[str, CTkFrame]
    current_view: str = "instances"

    def __init__(self) -> None:
        super().__init__()

        self.views = {
            "instances": Instances(master=self),
            "news": News(master=self),
            "accounts": Accounts(master=self),
            "settings": Settings(master=self),
            "about": About(master=self),
        }

        self.title("Ice Launcher")
        self.geometry(f"{App.WIDTH}x{App.HEIGHT}")
        self.protocol("WM_DELETE_WINDOW", self.on_closing)

        # configure grid layout (2x1)
        self.grid_columnconfigure(1, weight=1)
        self.grid_rowconfigure(0, weight=1)

        self.navigator = CTkFrame(master=self, width=180, corner_radius=0)  # type: ignore
        self.navigator.grid(row=0, column=0, sticky="nswe")

        self.instances_button = CTkButton(
            master=self.navigator,
            text="Instances",
            command=lambda: self.open_view("instances"),
            border_width=2,
            fg_color=None,
        )
        self.instances_button.grid(
            row=0, column=0, pady=(20, 10), padx=20, sticky="nswe"
        )

        self.news_button = CTkButton(
            master=self.navigator,
            text="News",
            command=lambda: self.open_view("news"),
            border_width=2,
            fg_color=None,
        )
        self.news_button.grid(row=1, column=0, pady=10, padx=20)

        self.accounts_button = CTkButton(
            master=self.navigator,
            text="Accounts",
            command=lambda: self.open_view("accounts"),
            border_width=2,
            fg_color=None,
        )
        self.accounts_button.grid(row=2, column=0, pady=10, padx=20)

        self.settings_button = CTkButton(
            master=self.navigator,
            text="Settings",
            command=lambda: self.open_view("settings"),
            border_width=2,
            fg_color=None,
        )
        self.settings_button.grid(row=3, column=0, pady=10, padx=20)

        # empty row as spacing
        self.navigator.grid_rowconfigure(4, weight=1)

        self.about_button = CTkButton(
            master=self.navigator,
            text="About",
            command=lambda: self.open_view("about"),
            border_width=2,
            fg_color=None,
        )
        self.about_button.grid(row=5, column=0, pady=(10, 20), padx=20)

        # place all views in the same place
        for view in self.views.values():
            view.grid(row=0, column=1, pady=20, padx=20, sticky="nswe")

        self.update_main_frame()

        if config.read()["automatically_check_for_updates"]:
            Thread(target=self.check_for_updates).start()

    def on_closing(self, event=0) -> None:
        self.destroy()

    def update_main_frame(self) -> None:
        self.views[self.current_view].tkraise()
        self.__dict__[f"{self.current_view}_button"].configure(fg_color="#1F6AA5")

    def open_view(self, view: str) -> None:
        self.__dict__[f"{self.current_view}_button"].configure(fg_color=None)
        self.current_view = view
        self.update_main_frame()

    def check_for_updates(self) -> None:
        latest_version = updater.check_for_updates(__version__)
        if latest_version:
            Update(master=self, latest_version=latest_version)


if __name__ == "__main__":
    app: App = App()
    app.mainloop()



# SPDX-FileCopyrightText: 2022-present Manuel Quarneti <manuelquarneti@gmail.com>
#
# SPDX-License-Identifier: GPL-3.0-only

import json
from os import path
from typing import List, TypedDict

from minecraft_launcher_lib.microsoft_types import CompleteLoginResponse

from . import dirs

__accounts_file__: str = path.join(dirs.user_data_dir, "accounts.json")


class Document(TypedDict):
    version: int
    accounts: List[CompleteLoginResponse]


def new_document() -> Document:
    doc: Document = {"version": 1, "accounts": []}

    write_document(doc)

    return doc


def read_document() -> Document:
    if not path.exists(__accounts_file__):
        doc = new_document()

    with open(__accounts_file__, "r") as f:
        doc = json.load(f)

    return doc


def write_document(doc: Document) -> None:
    with open(__accounts_file__, "w") as f:
        json.dump(doc, f)



# SPDX-FileCopyrightText: 2022-present Manuel Quarneti <manuelquarneti@gmail.com>
#
# SPDX-License-Identifier: GPL-3.0-only

from customtkinter import BooleanVar, CTkButton, CTkFrame, CTkLabel, CTkSwitch

from .lib import config


class Settings(CTkFrame):
    def __init__(self, master) -> None:
        super().__init__(master=master)

        self.app_config = config.read()

        self.grid_columnconfigure(0, weight=1)

        self.title_frame = CTkFrame(master=self, fg_color="gray38")
        self.title_frame.grid(row=0, column=0, pady=20, padx=20, sticky="nswe")

        self.view_name = CTkLabel(
            master=self.title_frame,
            text_font=("Roboto Medium", 30),
            text="Settings",
        )
        self.view_name.grid(row=0, column=0, pady=20, padx=20, sticky="nswe")

        self.automatically_check_for_updates = BooleanVar()

        self.automatically_check_for_updates_switch = CTkSwitch(
            master=self,
            text="Automatically check for updates",
            variable=self.automatically_check_for_updates,
        )
        self.automatically_check_for_updates_switch.grid(
            row=1, column=0, pady=10, padx=20, sticky="w"
        )

        # empty row as spacing
        self.grid_rowconfigure(2, weight=1)

        self.button_bar = CTkFrame(master=self)
        self.button_bar.grid(row=3, column=0, pady=0, padx=0, sticky="nswe")

        # empty column as spacing
        self.button_bar.grid_columnconfigure(0, weight=1)

        self.reset_button = CTkButton(
            master=self.button_bar,
            text="Reset to default settings",
            command=self.reset_to_default_settings,
        )
        self.reset_button.grid(row=0, column=1, pady=10, padx=10, sticky="se")

        self.save_button = CTkButton(
            master=self.button_bar,
            text="Save",
            command=self.save,
        )
        self.save_button.grid(row=0, column=2, pady=10, padx=10, sticky="se")

        self.update_settings()

    def update_settings(self) -> None:
        self.automatically_check_for_updates.set(
            self.app_config["automatically_check_for_updates"]
        )

    def reset_to_default_settings(self) -> None:
        self.app_config = config.default()
        self.update_settings()

    def save(self) -> None:
        self.app_config[
            "automatically_check_for_updates"
        ] = self.automatically_check_for_updates.get()

        config.write(self.app_config)



# SPDX-FileCopyrightText: 2022-present Manuel Quarneti <manuelquarneti@gmail.com>
#
# SPDX-License-Identifier: GPL-3.0-only

import tkinter

from customtkinter import CTkFrame, CTkLabel

from .__about__ import __version__


class About(CTkFrame):
    def __init__(self, master) -> None:
        super().__init__(master=master)

        self.grid_columnconfigure(0, weight=1)

        self.title_frame = CTkFrame(master=self, fg_color="gray38")
        self.title_frame.grid(row=0, column=0, pady=20, padx=20, sticky="nswe")

        self.app_name = CTkLabel(
            master=self.title_frame,
            text_font=("Roboto Medium", 30),
            text="Ice Launcher",
        )
        self.app_name.grid(row=0, column=0, pady=20, padx=20, sticky="nswe")

        self.info_frame = CTkFrame(master=self)
        self.info_frame.grid(row=1, column=0, pady=20, padx=20, sticky="nswe")
        self.info_frame.grid_columnconfigure(1, weight=1)

        self.app_info = CTkLabel(
            master=self.info_frame,
            text=f"Version {__version__}\n\nCopyright © 2022-present Manuel Quarneti\n\nGPL-3.0 Licensed",
            justify=tkinter.LEFT,
        )
        self.app_info.grid(row=1, column=0, pady=20, padx=20, sticky="w")



# SPDX-FileCopyrightText: 2022-present Manuel Quarneti <manuelquarneti@gmail.com>
#
# SPDX-License-Identifier: GPL-3.0-only

import json
from os import path
from typing import TypedDict

from . import dirs

__config_path__: str = path.join(dirs.user_data_dir, "config.json")


class Config(TypedDict):
    config_version: int
    automatically_check_for_updates: bool
    last_used_account: str


def default() -> Config:
    return {
        "config_version": 1,
        "automatically_check_for_updates": True,
        "last_used_account": "",
    }


def write(config: Config) -> None:
    with open(__config_path__, "w") as f:
        json.dump(config, f)


def read() -> Config:
    if not path.exists(__config_path__):
        write(default())

    with open(__config_path__, "r") as f:
        return json.load(f)



# SPDX-FileCopyrightText: 2022-present Manuel Quarneti <manuelquarneti@gmail.com>
#
# SPDX-License-Identifier: GPL-3.0-only

from .about import About
from .accounts import Accounts
from .instances import Instances
from .news import News
from .settings import Settings
from .update import Update

__all__ = ["About", "Accounts", "Instances", "News", "Settings", "Update"]



# SPDX-FileCopyrightText: 2022-present Manuel Quarneti <manuelquarneti@gmail.com>
#
# SPDX-License-Identifier: GPL-3.0-only

from os import makedirs, path

from appdirs import AppDirs

dirs = AppDirs("ice-launcher", "mq1.eu")
if not path.exists(dirs.user_data_dir):
    makedirs(dirs.user_data_dir)



# SPDX-FileCopyrightText: 2022-present Manuel Quarneti <manuelquarneti@gmail.com>
#
# SPDX-License-Identifier: GPL-3.0-only

import sys
import webbrowser

from customtkinter import CTkButton, CTkLabel, CTkToplevel

from .__about__ import __version__


class Update(CTkToplevel):
    def __init__(self, master, latest_version: str) -> None:
        super().__init__(master=master)
        self.latest_version = latest_version
        self.title("Update")

        self.grid_columnconfigure(0, weight=1)

        self.current_release = CTkLabel(
            master=self, text=f"Current version: {__version__}"
        )
        self.current_release.grid(
            row=0, column=0, columnspan=2, pady=(20, 10), padx=20, sticky="nswe"
        )

        self.latest_release = CTkLabel(
            master=self, text=f"Latest version: {latest_version}"
        )
        self.latest_release.grid(
            row=1, column=0, columnspan=2, pady=10, padx=20, sticky="nswe"
        )

        # empty row as spacing
        self.grid_rowconfigure(2, weight=1)

        self.update_question = CTkLabel(master=self, text="Do you want to update?")
        self.update_question.grid(row=3, column=0, columnspan=2, pady=(10, 20), padx=20)

        self.no_button = CTkButton(
            master=self, text="No", command=self.destroy, fg_color=None
        )
        self.no_button.grid(row=4, column=0, pady=10, padx=20)

        self.yes_button = CTkButton(
            master=self,
            text="Update",
            command=self.update,
        )
        self.yes_button.grid(row=4, column=1, pady=10, padx=20)

    def update(self) -> None:
        webbrowser.open(
            f"https://github.com/mq1/ice-launcher/releases/tag/{self.latest_version}"
        )
        sys.exit()



# SPDX-FileCopyrightText: 2022-present Manuel Quarneti <manuelquarneti@gmail.com>
#
# SPDX-License-Identifier: GPL-3.0-only

from tkinter import ttk

from customtkinter import CTkButton, CTkComboBox, CTkFrame, CTkLabel, StringVar

from .__about__ import __version__
from .components.scrollable_frame import ScrollableFrame
from .lib import accounts, config, instances
from .new_instance import NewInstance


class Instances(CTkFrame):
    def __init__(self, master) -> None:
        super().__init__(master=master)

        self.account_list = [a["name"] for a in accounts.read_document()["accounts"]]

        self.grid_columnconfigure(0, weight=1)

        self.title_frame = CTkFrame(master=self, fg_color="gray38")
        self.title_frame.grid(row=0, column=0, pady=20, padx=20, sticky="nswe")

        self.view_name = CTkLabel(
            master=self.title_frame,
            text_font=("Roboto Medium", 30),
            text="Instances",
        )
        self.view_name.grid(row=0, column=0, pady=20, padx=20, sticky="nswe")

        self.instances_list = ScrollableFrame(master=self)
        self.instances_list.grid(row=1, column=0, pady=20, padx=20, sticky="nswe")
        self.grid_rowconfigure(1, weight=1)
        self.instances_list.content.grid_columnconfigure(0, weight=1)

        self.status_bar = CTkFrame(master=self)
        self.status_bar.grid(row=2, column=0, pady=20, padx=20, sticky="nswe")

        # empty column as spacing
        self.status_bar.grid_columnconfigure(0, weight=1)

        self.account_label = CTkLabel(master=self.status_bar, text="Account:")
        self.account_label.grid(row=0, column=1, pady=0, padx=0, sticky="nse")

        self.selected_account = StringVar()
        self.account_selector = CTkComboBox(
            master=self.status_bar,
            values=self.account_list,
            command=lambda _: self.update_account_list(),
            variable=self.selected_account,
        )
        self.account_selector.grid(row=0, column=2, pady=10, padx=(0, 10), sticky="se")

        self.update_instance_list()
        self.set_default_account()
        self.update_account_list()

    def add_new_instance(self) -> None:
        self.new_instance_window = NewInstance(master=self)
        self.new_instance_window.protocol(
            "WM_DELETE_WINDOW", self.on_closing_new_instance_window
        )

    def on_closing_new_instance_window(self, event=0) -> None:
        self.new_instance_window.destroy()
        self.update_instance_list()

    def update_instance_list(self) -> None:
        instance_list = instances.list()

        for instance in self.instances_list.content.winfo_children():
            instance.destroy()

        for index, instance_name in enumerate(instance_list):
            label = CTkLabel(master=self.instances_list.content, text=instance_name)
            label.grid(row=index * 2, column=0, pady=10, padx=10, sticky="nw")
            launch_button = CTkButton(
                master=self.instances_list.content,
                text="Launch",
                command=lambda: instances.launch(
                    instance_name, self.selected_account.get(), __version__
                ),
            )
            launch_button.grid(row=index * 2, column=1, pady=10, padx=10, sticky="e")
            separator = ttk.Separator(self.instances_list.content, orient="horizontal")
            separator.grid(
                row=index * 2 + 1, column=0, columnspan=2, pady=0, padx=10, sticky="ew"
            )

        new_instance_button = CTkButton(
            master=self.instances_list.content,
            text="New Instance",
            command=self.add_new_instance,
        )
        new_instance_button.grid(
            row=len(instance_list) * 2, column=0, pady=20, padx=20, sticky="nw"
        )

    def set_default_account(self) -> None:
        c = config.read()
        if c["last_used_account"] != "":
            self.selected_account.set(c["last_used_account"])
        elif len(self.account_list) > 0:
            self.selected_account.set(self.account_list[0])

    def update_account_list(self) -> None:
        self.account_list = [a["name"] for a in accounts.read_document()["accounts"]]
        self.account_selector.configure(values=self.account_list)

        # update last used account
        c = config.read()
        c["last_used_account"] = self.selected_account.get()
        config.write(c)
